export declare const defaultConfigPath: string;
export type ConfigEntry = {
    token: string;
    username: string;
};
export type ClientConfig = {
    'api.heroku.com': ConfigEntry;
    'git.heroku.com': ConfigEntry;
    [machineId: string]: ConfigEntry;
};
/**
 * Reads the contents of the config file and decrypts it using the provided bearer token
 * and returns the JSON contents of the config file.
 *
 * @param token the bearer token to use for encryption
 * @param configPath the path to the config file to write
 * @returns a promise that resolves to the JSON contents of the config file
 */
export declare function getConfigContents(token: string, configPath?: string): Promise<ClientConfig>;
/**
 * Writes the provided contents to the config file and encrypts it using the provided bearer token.
 *
 * @param contents the content to write to the config file. This must be a JSON serializable object
 * @param token the bearer token to use for encryption
 * @param configPath the path to the config file to write
 * @returns a promise that resolves when the config file has been written
 */
export declare function writeConfigContents(contents: unknown, token: string, configPath?: string): Promise<void>;
/**
 * Derives a key using the provided bearer token and salt.
 * This function is critical to ensuring the key is permanently
 * bound to a unique bearer token obtained via secure authentication
 * flows. If the bearer token is revoked, the encrypted data is
 * intentionally not recoverable.
 *
 * @param bearerToken The bearer token to use for encryption. This must be provided as a revokable bearer token
 * @param salt The salt to use for key derivation. This must be a buffer of 16 bytes
 * @returns the derived key as a buffer
 */
export declare function deriveKey(bearerToken: string, salt: Buffer): Buffer;
/**
 * Decrypts the given encrypted content using the provided bearer token.
 * The encrypted content is expected to be in the format:
 * salt:iv:authTag:encryptedContent
 *
 * @param encrypted the content to decrypt
 * @param bearerToken the bearer token to use for decryption
 * @returns the decrypted content as a string
 */
export declare function decryptWithToken(encrypted: string, bearerToken: string): string;
