"use strict";
/**
 * Functions for securely storing bearer tokens using OS-level utilities.
 * These functions are limited to credentials and do not provide general
 * encryption for arbitrary data.
 *
 * If you need to store and retrieve sensitive information other
 * than credentials, you should consider using the utilities provided by
 * `file-encryption.ts` instead.
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.SERVICE_NAME = void 0;
exports.removeToken = removeToken;
exports.storeToken = storeToken;
exports.retrieveToken = retrieveToken;
const tslib_1 = require("tslib");
const child_process_1 = require("child_process");
const os = tslib_1.__importStar(require("os"));
const crypto = tslib_1.__importStar(require("crypto"));
exports.SERVICE_NAME = 'heroku';
/**
 * Derives a unique token name based on the service name and machine/user info
 *
 * @param serviceName - The name of the service for which the token is being stored
 * @returns A derived token name based on the service name and machine/user info
 */
function deriveTokenName(serviceName = exports.SERVICE_NAME) {
    // Use only deterministic components
    // but ensure uniqueness across services and machines
    // This is an added layer of protection to prevent token collisions
    // and guarantees the token name is not easily guessable
    const components = [
        serviceName, // Service name
        os.hostname(), // Machine hostname
        os.userInfo().username, // OS username
    ];
    // Create a hash of the combined components
    // to ensure the token length is known and consistent
    const hash = crypto.createHash('sha256')
        .update(components.join('::'))
        .digest('hex')
        .slice(0, 32);
    return `${serviceName}_${hash}`;
}
/**
 * Stores a bearer token securely on macOS using the Keychain
 * @param tokenName - Unique identifier/name for the token
 * @param tokenValue - The bearer token value to store
 * @throws Error if the token cannot be stored
 * @returns void
 */
function storeTokenMacOS(tokenName, tokenValue) {
    try {
        try {
            (0, child_process_1.execSync)(`security delete-generic-password -a "${tokenName}" -s "bearer_token"`);
        }
        catch (_a) {
            // noop - item does not exist
        }
        (0, child_process_1.execSync)(`security add-generic-password -a "${tokenName}" -s "bearer_token" -w "${tokenValue}"`);
    }
    catch (error) {
        const { message } = error;
        throw new Error(`Failed to store token in macOS Keychain: ${message}`);
    }
}
/**
 * Stores a bearer token securely on Windows using Windows Credential Manager
 * @param tokenName - Unique identifier/name for the token
 * @param tokenValue - The bearer token value to store
 * @throws Error if the token cannot be stored
 * @returns void
 */
function storeTokenWindows(tokenName, tokenValue) {
    try {
        try {
            const removeCommand = `
        [void][Windows.Security.Credentials.PasswordVault,Windows.Security.Credentials,ContentType=WindowsRuntime]
        $vault = New-Object Windows.Security.Credentials.PasswordVault
        $credential = $vault.Retrieve("${tokenName}", "${tokenName}")
        $vault.Remove($credential)
      `;
            (0, child_process_1.execSync)(removeCommand, { shell: 'powershell' });
        }
        catch (_a) {
            // noop - item does not exist
        }
        const addCommand = `
      [void][Windows.Security.Credentials.PasswordVault,Windows.Security.Credentials,ContentType=WindowsRuntime]
      $vault = New-Object Windows.Security.Credentials.PasswordVault
      $credential = New-Object Windows.Security.Credentials.PasswordCredential("${tokenName}", "${tokenName}", "${tokenValue}")
      $vault.Add($credential)
    `;
        (0, child_process_1.execSync)(addCommand, { shell: 'powershell' });
    }
    catch (error) {
        const { message } = error;
        throw new Error(`Failed to store token in Windows Credential Manager: ${message}`);
    }
}
/**
 * Stores a bearer token securely on Linux using the system keyring (via secret-tool)
 * @param tokenName - Unique identifier/name for the token
 * @param tokenValue - The bearer token value to store
 * @throws Error if the token cannot be stored or if secret-tool is not installed
 * @returns void
 */
function storeTokenLinux(tokenName, tokenValue) {
    try {
        try {
            (0, child_process_1.execSync)(`secret-tool clear type bearer_token name "${tokenName}"`);
        }
        catch (_a) {
            // noop - item does not exist
        }
        (0, child_process_1.execSync)(`echo "${tokenValue}" | secret-tool store --label="${tokenName}" type bearer_token name "${tokenName}"`);
    }
    catch (error) {
        const { message } = error;
        throw new Error(`Failed to store token in Linux keyring: ${message}. Make sure secret-tool is installed.`);
    }
}
/**
 * Retrieves a stored bearer token on macOS from the Keychain
 * @param tokenName - Unique identifier/name of the token to retrieve
 * @throws Error if the token cannot be retrieved
 * @returns void
 */
function retrieveTokenMacOS(tokenName) {
    try {
        const output = (0, child_process_1.execSync)(`security find-generic-password -a "${tokenName}" -s "bearer_token" -w`);
        return output.toString().trim();
    }
    catch (error) {
        const { message } = error;
        throw new Error(`Failed to retrieve token from macOS Keychain: ${message}`);
    }
}
/**
 * Retrieves a stored bearer token on Windows from the Credential Manager
 * @param tokenName - Unique identifier/name of the token to retrieve
 * @throws Error if the token cannot be retrieved
 * @returns void
 */
function retrieveTokenWindows(tokenName) {
    try {
        const psCommand = `
      [void][Windows.Security.Credentials.PasswordVault,Windows.Security.Credentials,ContentType=WindowsRuntime]
      $vault = New-Object Windows.Security.Credentials.PasswordVault
      $credential = $vault.Retrieve("${tokenName}", "${tokenName}")
      $credential.Password
    `;
        const output = (0, child_process_1.execSync)(psCommand, { shell: 'powershell' });
        const token = output.toString().trim();
        if (!token) {
            throw new Error('Token not found');
        }
        return token;
    }
    catch (error) {
        const { message } = error;
        throw new Error(`Failed to retrieve token from Windows Credential Manager: ${message}`);
    }
}
/**
 * Retrieves a stored bearer token on Linux from the system keyring
 *
 * @param tokenName - Unique identifier/name of the token to retrieve
 * @throws Error if the token cannot be retrieved
 * @returns void
 */
function retrieveTokenLinux(tokenName) {
    try {
        const output = (0, child_process_1.execSync)(`secret-tool lookup type bearer_token name "${tokenName}"`);
        return output.toString().trim();
    }
    catch (error) {
        const { message } = error;
        throw new Error(`Failed to retrieve token from Linux keyring: ${message}`);
    }
}
/**
 * Removes a stored bearer token from macOS Keychain
 *
 * @param tokenName - Unique identifier/name of the token to remove
 * @throws Error if the token cannot be removed or doesn't exist
 * @returns void
 */
function removeTokenMacOS(tokenName) {
    try {
        (0, child_process_1.execSync)(`security delete-generic-password -a "${tokenName}" -s "bearer_token"`);
    }
    catch (error) {
        const { message } = error;
        throw new Error(`Failed to remove token from macOS Keychain: ${message}`);
    }
}
/**
 * Removes a stored bearer token from Windows Credential Manager
 *
 * @param tokenName - Unique identifier/name of the token to remove
 * @throws Error if the token cannot be removed or doesn't exist
 * @returns void
 */
function removeTokenWindows(tokenName) {
    try {
        const psCommand = `
      [void][Windows.Security.Credentials.PasswordVault,Windows.Security.Credentials,ContentType=WindowsRuntime]
      $vault = New-Object Windows.Security.Credentials.PasswordVault
      $credential = $vault.Retrieve("${tokenName}", "${tokenName}")
      $vault.Remove($credential)
    `;
        (0, child_process_1.execSync)(psCommand, { shell: 'powershell' });
    }
    catch (error) {
        const { message } = error;
        throw new Error(`Failed to remove token from Windows Credential Manager: ${message}`);
    }
}
/**
 * Removes a stored bearer token from Linux system keyring
 *
 * @param tokenName - Unique identifier/name of the token to remove
 * @throws Error if the token cannot be removed or doesn't exist
 * @returns void
 */
function removeTokenLinux(tokenName) {
    try {
        (0, child_process_1.execSync)(`secret-tool clear type bearer_token name "${tokenName}"`);
    }
    catch (error) {
        const { message } = error;
        throw new Error(`Failed to remove token from Linux keyring: ${message}`);
    }
}
/**
 * Removes a stored bearer token using OS-level utilities
 *
 * @param serviceName - The name of the service for which the token is being removed. Defaults to 'heroku'
 * @throws Error if the token cannot be removed or if the platform is not supported
 * @returns Promise that resolves when the token is removed
 *
 * @example
 * ```typescript
 * // Remove token for default service (heroku)
 * removeToken();
 *
 * // Remove token for a specific service
 * removeToken('my-service');
 * ```
 */
function removeToken(serviceName) {
    const tokenName = deriveTokenName(serviceName);
    switch (process.platform) {
        case 'win32':
            removeTokenWindows(tokenName);
            break;
        case 'darwin':
            removeTokenMacOS(tokenName);
            break;
        case 'linux':
            removeTokenLinux(tokenName);
            break;
        default:
            throw new Error(`Unsupported platform: ${process.platform}`);
    }
}
/**
 * Stores a bearer token securely using OS-level utilities
 *
 * @param token the token to store
 * @param serviceName the name of the service for which the token is being stored. defaults to 'heroku'
 * @returns a promise that resolves when the token is stored
 */
function storeToken(token, serviceName) {
    const tokenName = deriveTokenName(serviceName);
    switch (process.platform) {
        case 'win32':
            storeTokenWindows(tokenName, token);
            break;
        case 'darwin':
            storeTokenMacOS(tokenName, token);
            break;
        case 'linux':
            storeTokenLinux(tokenName, token);
            break;
        default:
            throw new Error(`Unsupported platform: ${process.platform}`);
    }
}
/**
 * Retrieves a stored bearer token securely using OS-level utilities
 *
 * @param serviceName the name of the service for which the token is being stored. defaults to 'heroku'
 * @returns a promise that resolves with the stored token
 */
function retrieveToken(serviceName) {
    const tokenName = deriveTokenName(serviceName);
    switch (process.platform) {
        case 'win32':
            return retrieveTokenWindows(tokenName);
        case 'darwin':
            return retrieveTokenMacOS(tokenName);
        case 'linux':
            return retrieveTokenLinux(tokenName);
        default:
            throw new Error(`Unsupported platform: ${process.platform}`);
    }
}
